@@ -5,6 +5,7 @@
  * Copyright (C) 2003: Helio Chissini de Castro <helio@conectiva.com>
  * Copyright (C) 2007 Henrique Pinto <henrique.pinto@kdemail.net>
  * Copyright (C) 2008 Harald Hvaal <haraldhv@stud.ntnu.no>
+ * Copyright (C) 2021 Jiří Wolker <woljiri@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,6 +31,7 @@
 #include "settingspage.h"
 #include "pluginmanager.h"
 #include "interface.h"
+#include "welcomescreen.h"
 
 #include <KParts/ReadWritePart>
 #include <KPluginFactory>
@@ -37,7 +39,7 @@
 #include <KLocalizedString>
 #include <KActionCollection>
 #include <KStandardAction>
-#include <KRecentFilesAction>
+#include <KRecentFilesMenu>
 #include <KSharedConfig>
 #include <KConfigDialog>
 #include <KXMLGUIFactory>
@@ -51,6 +53,7 @@
 #include <QMimeData>
 #include <QPointer>
 #include <QStatusBar>
+#include <QStackedWidget>
 
 static bool isValidArchiveDrag(const QMimeData *data)
 {
@@ -59,22 +62,25 @@ static bool isValidArchiveDrag(const QMimeData *data)
 
 MainWindow::MainWindow(QWidget *)
         : KParts::MainWindow()
+        , m_welcomeScreen(new WelcomeScreen(this))
+        , m_windowContents(new QStackedWidget(this))
 {
     setupActions();
     setAcceptDrops(true);
     // Ark doesn't provide a fullscreen mode; remove the corresponding window button
     setWindowFlags(windowFlags() & ~Qt::WindowFullscreenButtonHint);
+
+    setCentralWidget(m_windowContents);
+    m_windowContents->addWidget(m_welcomeScreen);
+    showWelcomeScreen();
 }
 
 MainWindow::~MainWindow()
 {
-    if (m_recentFilesAction) {
-        m_recentFilesAction->saveEntries(KSharedConfig::openConfig()->group("Recent Files"));
-    }
-
     guiFactory()->removeClient(m_part);
     delete m_part;
     m_part = nullptr;
+    m_welcomeScreen = nullptr;
 }
 
 void MainWindow::dragEnterEvent(QDragEnterEvent * event)
@@ -129,17 +135,7 @@ void MainWindow::dragMoveEvent(QDragMoveEvent * event)
 
 bool MainWindow::loadPart()
 {
-    KPluginFactory *factory = nullptr;
-
-    const auto plugins = KPluginLoader::findPlugins(QString(), [](const KPluginMetaData& metaData) {
-        return metaData.pluginId() == QLatin1String("arkpart") &&
-               metaData.serviceTypes().contains(QLatin1String("KParts/ReadOnlyPart")) &&
-               metaData.serviceTypes().contains(QLatin1String("Browser/View"));
-    });
-
-    if (!plugins.isEmpty()) {
-        factory = KPluginLoader(plugins.first().fileName()).factory();
-    }
+    KPluginFactory *factory = KPluginLoader(QStringLiteral("kf5/parts/arkpart")).factory();
 
     m_part = factory ? static_cast<KParts::ReadWritePart*>(factory->create<KParts::ReadWritePart>(this)) : nullptr;
 
@@ -150,7 +146,7 @@ bool MainWindow::loadPart()
     }
 
     m_part->setObjectName(QStringLiteral("ArkPart"));
-    setCentralWidget(m_part->widget());
+    m_windowContents->addWidget(m_part->widget());
 
     setXMLFile(QStringLiteral("arkui.rc"));
     setupGUI(ToolBar | Keys | Save);
@@ -159,6 +155,7 @@ bool MainWindow::loadPart()
     statusBar()->hide();
 
     connect(m_part, SIGNAL(ready()), this, SLOT(updateActions()));
+    connect(m_part, SIGNAL(ready()), this, SLOT(hideWelcomeScreen()));
     connect(m_part, SIGNAL(quit()), this, SLOT(quit()));
     // #365200: this will disable m_recentFilesAction, while openUrl() will enable it.
     // So updateActions() needs to be called after openUrl() returns.
@@ -170,6 +167,17 @@ bool MainWindow::loadPart()
     return true;
 }
 
+void MainWindow::showWelcomeScreen()
+{
+    m_windowContents->setCurrentWidget(m_welcomeScreen);
+}
+
+void MainWindow::hideWelcomeScreen()
+{
+    Q_ASSERT(m_part->widget());
+    m_windowContents->setCurrentWidget(m_part->widget());
+}
+
 void MainWindow::setupActions()
 {
     m_newAction = KStandardAction::openNew(this, &MainWindow::newArchive, this);
@@ -178,16 +186,15 @@ void MainWindow::setupActions()
     actionCollection()->addAction(QStringLiteral("ark_file_open"), m_openAction);
     auto quitAction = KStandardAction::quit(this, &MainWindow::quit, this);
     actionCollection()->addAction(QStringLiteral("ark_quit"), quitAction);
-    m_recentFilesAction = KStandardAction::openRecent(this, &MainWindow::openUrl, this);
-    actionCollection()->addAction(QStringLiteral("ark_file_open_recent"), m_recentFilesAction);
-
-    m_recentFilesAction->setToolBarMode(KRecentFilesAction::MenuMode);
-    m_recentFilesAction->setToolButtonPopupMode(QToolButton::DelayedPopup);
-    m_recentFilesAction->setIconText(i18nc("action, to open an archive", "Open"));
-    m_recentFilesAction->setToolTip(i18n("Open an archive"));
-    m_recentFilesAction->loadEntries(KSharedConfig::openConfig()->group("Recent Files"));
+    m_recentFilesMenu = new KRecentFilesMenu(this);
+    actionCollection()->addAction(QStringLiteral("ark_file_open_recent"), m_recentFilesMenu->menuAction());
+    connect(m_recentFilesMenu, &KRecentFilesMenu::urlTriggered, this, &MainWindow::openUrl);
 
     KStandardAction::preferences(this, &MainWindow::showSettings, actionCollection());
+
+    // Connect the welcome screen to actions created above
+    connect(m_welcomeScreen, &WelcomeScreen::newClicked, m_newAction, &QAction::trigger);
+    connect(m_welcomeScreen, &WelcomeScreen::openClicked, m_openAction, &QAction::trigger);
 }
 
 void MainWindow::updateActions()
@@ -196,7 +203,7 @@ void MainWindow::updateActions()
     Kerfuffle::PluginManager pluginManager;
     m_newAction->setEnabled(!iface->isBusy() && !pluginManager.availableWritePlugins().isEmpty());
     m_openAction->setEnabled(!iface->isBusy());
-    m_recentFilesAction->setEnabled(!iface->isBusy());
+    m_recentFilesMenu->setEnabled(!iface->isBusy());
 }
 
 void MainWindow::openArchive()
@@ -254,6 +261,12 @@ void MainWindow::closeEvent(QCloseEvent *event)
     KParts::MainWindow::closeEvent(event);
 }
 
+// Set a sane default window size
+QSize MainWindow::sizeHint() const
+{
+    return QSize(700, 500);
+}
+
 void MainWindow::quit()
 {
     close();
@@ -294,7 +307,7 @@ void MainWindow::writeSettings()
 
 void MainWindow::addPartUrl()
 {
-    m_recentFilesAction->addUrl(m_part->url());
+    m_recentFilesMenu->addUrl(m_part->url());
 }
 
 void MainWindow::newArchive()
